/*
 *  Switcher.cs: a tray applet for dynamically changing the XFree86 display mode
 *  Copyright 2003 Alp Toker <alp@atoker.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

using System;
using System.IO;
using System.Collections;
using System.Diagnostics;
using System.Text.RegularExpressions;
using Gtk;
using GtkSharp;
using Egg;

public class Switcher
{
	const string switchto = "/usr/bin/switchto";
	static Menu menu;
	static Hashtable modes;
	static string contents;

	static void Main ()
	{
		Application.Init ();

		modes = new Hashtable ();
		menu = new Menu ();

		StreamReader sr = new StreamReader(@"/etc/switchto.last");
		contents = sr.ReadLine();
		Console.WriteLine(contents);
		
		ProcessStartInfo psi = new ProcessStartInfo (switchto, "-l --nocolors");
		psi.RedirectStandardOutput = true;
		Process p = Process.Start (psi);

		Regex re_mode = new Regex(" - ");
		Regex before = new Regex(".* - ");
		Regex after = new Regex(" -.*");
		
		GLib.SList mode_group = new GLib.SList (IntPtr.Zero);

		foreach (string l in p.StandardOutput.ReadToEnd ().Split ('\n')) {
			//Console.WriteLine(l);
			Match m = re_mode.Match (l);
			if (m.Success) {
				//RadioMenuItem it = new RadioMenuItem (mode_group, m.Groups[3].ToString ());
				RadioMenuItem it = new RadioMenuItem (mode_group, before.Replace(l,""));
				mode_group = it.Group;
				//modes[it] = Convert.ToInt32 (m.Groups[2].ToString ());
				modes[it] = after.Replace(l,"");
				//if (m.Groups[1].ToString ().Trim () == "*") it.Active = true;
				if (contents == modes[it].ToString())
				{
					Console.WriteLine("hit");
					it.Active = true;
				}
				it.Activated += new EventHandler (ModeSelected);
				menu.Append (it);
				continue;
			}
		}

		menu.Append (new SeparatorMenuItem ());

		ImageMenuItem it_quit  = new ImageMenuItem ("gtk-quit", new Gtk.AccelGroup ());
		it_quit.Activated += new EventHandler (QuitSelected);
		menu.Append (it_quit);

		EventBox eb = new EventBox ();
		eb.ButtonPressEvent += new ButtonPressEventHandler (IconClicked);
		eb.Add (new Image (new Gdk.Pixbuf (null, "switcher.png")));

		TrayIcon t = new TrayIcon ("Network Configuration Switcher");
		t.Add (eb);
		t.ShowAll ();

		Application.Run ();
	}

	static void IconClicked (object o, ButtonPressEventArgs args)
	{
		menu.ShowAll ();
		menu.Popup (null, null, null, IntPtr.Zero, args.Event.button, args.Event.time);
	}

	static void ModeSelected (object o, EventArgs args)
	{
		if (!((RadioMenuItem) o).Active) return;
		Console.WriteLine(modes[o]);
		Process.Start (switchto, modes[o].ToString ());
	}

	static void QuitSelected (object o, EventArgs args)
	{
		Application.Quit ();
	}
}
